package mpi.dcr;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.ResourceBundle;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;


/**
 * A panel with user interface elements to interact with a locally stored  data
 * category selection.
 *
 * @author Han Sloetjes
 * @version 1.0
 * 
 * @author aarsom
 * @version 2.0 the JList is replaced with a JTable and the values are multilingual
 */
public class LocalDCSelectPanel extends AbstractDCSelectPanel2 {
    /** add categories button */
    protected JButton addCatsButton;

    /** remove categories button */
    protected JButton removeCatsButton;
    
    protected JPanel buttonPanel;
    
    private List<DCSmall> allDatCats;
   

    /**
     * Creates a new LocalDCSelectPanel instance
     *
     * @param connector the (local) connector to use
     */
    public LocalDCSelectPanel(ILATDCRConnector connector) {
        super(connector);

        if (connector instanceof LocalDCRConnector) {
            this.connector = (LocalDCRConnector) connector;
        }

        initialize();
    }

    /**
     * Creates a new LocalDCSelectPanel instance
     *
     * @param connector the connector
     * @param resBundle a resource bundle with localized strings
     */
    public LocalDCSelectPanel(ILATDCRConnector connector,
        ResourceBundle resBundle) {
        super(connector, resBundle);

        if (connector instanceof LocalDCRConnector) {
            this.connector = (LocalDCRConnector) connector;
        }

        initialize();
    }

    /**
     * Creates a new LocalDCSelectPanel instance
     */
    public LocalDCSelectPanel() {
        super();
        initialize();
    }

    /**
     * Initializes ui components.
     */
    protected void initComponents() {
        try {
            allDatCats = connector.getDCSmallList(null, null);

            if (allDatCats == null) {
                allDatCats = new ArrayList<DCSmall>();
            }
        } catch (DCRConnectorException dce) {
            allDatCats = new ArrayList<DCSmall>();
        }
        
        setLayout(new GridBagLayout());

        // categories panel
        addCatsButton = new JButton();
        removeCatsButton = new JButton();
        
        addCatsButton.addActionListener(this);
        removeCatsButton.addActionListener(this);

        buttonPanel = new JPanel(new GridBagLayout());
        
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.anchor = GridBagConstraints.CENTER;
        gbc.insets = super.globalInsets;
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.fill = GridBagConstraints.NONE;
        buttonPanel.add(addCatsButton, gbc);
        
        gbc.gridx = 1;
        buttonPanel.add(removeCatsButton, gbc);
       
        gbc = new GridBagConstraints();
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.insets = super.globalInsets;
        gbc.gridx = 0;
        gbc.weightx = 1.0;
        gbc.gridy = 1;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.weighty = 0.0;
        catPanel.add(buttonPanel, gbc);
        
        gbc = new GridBagConstraints();
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.insets = super.globalInsets;
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        add(catPanel, gbc);
        
        gbc.gridx = 1;
        gbc.fill = GridBagConstraints.VERTICAL;
        gbc.weightx = 0.0;
        add(descPanel, gbc);

        updateLocale();
        
        updateCategories(allDatCats);
    }

    /**
     * Applies localized texts to elements, if provided.
     */
    protected void updateLocale() {
        super.updateLocale();

        String selCat = "Select Category";
        String addCat = "Add Categories";
        String remCat = "Remove Categories";

        if (resBundle != null) {
        	try {
        		selCat = resBundle.getString("DCR.Label.SelectCategory");
        	} catch (Exception ex) {
        	}
            
        	try {
                addCat = resBundle.getString("DCR.Label.AddCategories");
            } catch (Exception ex) {
            }

            try {
                remCat = resBundle.getString("DCR.Label.RemoveCategories");
            } catch (Exception ex) {
            }
        }
        
        catPanel.setBorder(new TitledBorder(selCat));
        addCatsButton.setText(addCat);
        removeCatsButton.setText(remCat);
    }

  

    /**
     * Creates a dialog to connect to a remote DCR and adds the selected
     * categories to the cache.
     */
    protected void selectAndAddCategories() {
        //String dcrLoc = "http://lux12.mpi.nl/isocat/rpc/syntax";
        //String dcrName = "SYNTAX ISO12620";
        // accept the default
        /*
           LATDCRConnector remConnector = new LATDCRConnector();
           //remConnector.setDCRLocation(dcrLoc);
           //remConnector.setName(dcrName);
           RemoteDCSelectPanel dcsp = new RemoteDCSelectPanel(remConnector);
           //dcsp.addProfiles(pp.getSelectedProfileNames());
           JDialog catFrame = new JDialog();
           catFrame.setModal(true);
           catFrame.setContentPane(dcsp);
           catFrame.pack();
           catFrame.setVisible(true);// blocks
           List dcsToAdd = dcsp.getSelectedCategories();
           addCategories(dcsToAdd);
         */
    }

    /**
     * Adds the categories from the specified list to the local list.
     *
     * @param dcsToAdd the data categories to add
     */
    protected void addCategories(List dcsToAdd) {
        if ((dcsToAdd != null) && (dcsToAdd.size() > 0)) {
            // add the new categories
            List<DCSmall> addedDcs = new ArrayList<DCSmall>(dcsToAdd.size());
            DCSmall small1 = null;
            DCSmall small2 = null;
            
dcloop: 
            for (int i = 0; i < dcsToAdd.size(); i++) {
                small1 = (DCSmall) dcsToAdd.get(i);

                for (int j = 0; j < allDatCats.size(); j++) {
                    small2 = (DCSmall) allDatCats.get(j);

                    if (small1.getIdentifier().equals(small2.getIdentifier())) {
                        // already in the list, replace
                    	allDatCats.set(j, small1);
                        continue dcloop;
                    }
                }

                // not in the list
                addedDcs.add(small1);
            }

            if (addedDcs.size() > 0) {
                allDatCats.addAll(addedDcs);
                updateCategories(allDatCats);

                if (connector instanceof LocalDCRConnector) {
                    try {
                        ((LocalDCRConnector) connector).addDataCategories(addedDcs);
                    } catch (DCRConnectorException dce) {
                        // message
                    }
                }
            }
        }
    }

    private void removeCategories() {
    	List remDatCats = new ArrayList();
    	Object sel = null;

      	int[] rows =  catTable.getSelectedRows();
      	Arrays.sort(rows);
      	if(rows != null){
      		for(int i = rows.length-1; i >= 0; i--){
      			sel = catTable.getValueAt(rows[i], 0);;        			
      			if (sel instanceof DCSmall) {
      				allDatCats.remove(sel);
      				catModel.removeRow(rows[i]);
      				remDatCats.add(sel);
      			}
      		}
        }
        	
      	if (remDatCats.size() > 0 && connector instanceof LocalDCRConnector) {
      		try {
      			((LocalDCRConnector) connector).removeCategories(remDatCats);
      		} catch (DCRConnectorException dce) {
      			// message
      		}
      	}
    }

    /**
     * The action event handling.
     *
     * @param e the action event
     */
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == addCatsButton) {
            selectAndAddCategories();
        } else if (e.getSource() == removeCatsButton) {
            removeCategories();
        } else {
        	super.actionPerformed(e);
        }
    }

	
}
