package mpi.eudico.client.annotator.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.AbstractListModel;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import mpi.eudico.client.annotator.Constants;
import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.util.BasicControlledVocabulary;
import mpi.eudico.util.ControlledVocabulary;
import mpi.eudico.util.multilangcv.LangInfo;
import mpi.eudico.util.multilangcv.LanguageCollection;

@SuppressWarnings("serial")
public class EditCVLanguagesDialog extends JDialog implements ActionListener {
	private ControlledVocabulary cv;
    private JComboBox cvLanguageComboBox;
    private JComboBox cvNewLanguageComboBox;
    private LangInfo selectedNewLanguage;
	private JButton addButton;
	private JButton changeButton;
	private JButton deleteButton;
	private JButton closeButton;

	public EditCVLanguagesDialog(AbstractEditCVDialog parent, ControlledVocabulary cv) {
		super(parent, true);
		this.cv = cv;
		
		/*
		 * Since this dialog is meant to be modal, no locale changes are expected
		 * while it is open.
		 */
		setTitle(ElanLocale.getString("EditCVLanguagesDialog.Title"));
		makeLayout();
	}
	
	void makeLayout() {
		getContentPane().setLayout(new GridBagLayout());

        Insets insets = new Insets(6, 10, 6, 10);
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = GridBagConstraints.RELATIVE;
		gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.weightx = 1.0;
        gbc.anchor = GridBagConstraints.CENTER;
		gbc.insets = insets;
		GridBagConstraints gbc_center_x = (GridBagConstraints)gbc.clone();
        gbc_center_x.fill = GridBagConstraints.NONE;
		
		JLabel label = new JLabel(ElanLocale.getString("EditCVLanguagesDialog.Title"));
        label.setFont(label.getFont().deriveFont((float) 16));
        label.setMaximumSize(label.getPreferredSize());
        add(label, gbc_center_x);

		JPanel contentPanel = new JPanel();
		contentPanel.setLayout(new GridBagLayout());
		contentPanel.setBorder(new TitledBorder(ElanLocale.getString("EditCVLanguagesDialog.Border")));
		
		label = new JLabel(ElanLocale.getString("EditCVLanguagesDialog.Label.Available"));
		contentPanel.add(label, gbc);
        cvLanguageComboBox = new JComboBox();
        contentPanel.add(cvLanguageComboBox, gbc);

        label = new JLabel(ElanLocale.getString("EditCVLanguagesDialog.Label.Edit"));
        contentPanel.add(label, gbc);
        
        cvNewLanguageComboBox = new JComboBox();
        cvNewLanguageComboBox.setEditable(true);
        cvNewLanguageComboBox.setMaximumRowCount(Constants.COMBOBOX_VISIBLE_ROWS);
        contentPanel.add(cvNewLanguageComboBox, gbc);
        
        Box buttonBox = new Box(BoxLayout.X_AXIS);
        
        addButton = new JButton(ElanLocale.getString("EditCVLanguagesDialog.Button.Add"));
		buttonBox.add(addButton);

		changeButton = new JButton(ElanLocale.getString("EditCVLanguagesDialog.Button.Change"));
		buttonBox.add(changeButton);

		deleteButton = new JButton(ElanLocale.getString("EditCVLanguagesDialog.Button.Delete"));
		buttonBox.add(deleteButton);

		contentPanel.add(buttonBox, gbc_center_x);
		add(contentPanel, gbc);

		closeButton = new JButton(ElanLocale.getString("EditCVLanguagesDialog.Button.Close"));
		add(closeButton, gbc_center_x);
		
        cvLanguageComboBox.addActionListener(this);
        cvNewLanguageComboBox.addActionListener(this);
		addButton.addActionListener(this);
		changeButton.addActionListener(this);
		deleteButton.addActionListener(this);
		closeButton.addActionListener(this);

        updateLanguageComboBox();
        updateNewLanguageComboBox();

        pack();        
        setLocationRelativeTo(getParent());
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		
		if (source == addButton) {
			getSelectedNewLanguage();
			if (selectedNewLanguage == null) {
				showErrorDialog(ElanLocale.getString("EditCVLanguagesDialog.Error.NothingToAdd"));
				return;
			}
			String s = selectedNewLanguage.getId();
			String l = selectedNewLanguage.getLongId();
			String lab = selectedNewLanguage.getLabel();
			int newindex = cv.addLanguage(s, l, lab);
			
			if (newindex >= 0) {
				updateLanguageComboBox();
				cvLanguageComboBox.setSelectedIndex(newindex);				
			} else {
				showErrorDialog(ElanLocale.getString("EditCVLanguagesDialog.Error.AddFailed"));
			}
		} else if (source == deleteButton) {
			int index = cvLanguageComboBox.getSelectedIndex();
			if (index >= 0) {
				LangInfo item = (LangInfo)cvLanguageComboBox.getItemAt(index);
				String name = item.toString();
				String sure = String.format(ElanLocale.getString("EditCVLanguagesDialog.Confirm.Delete"),
						name); // the format contains an instance of "%s".
				if (showConfirmDialog(sure)) {
	    			cv.removeLanguage(index);
	    			updateLanguageComboBox();
				}
			}
		} else if (source == changeButton) {
			getSelectedNewLanguage();
			if (selectedNewLanguage == null) {
				showErrorDialog(ElanLocale.getString("EditCVLanguagesDialog.Error.NothingToChange"));
				return;
			}
			String s = selectedNewLanguage.getId();
			String l = selectedNewLanguage.getLongId();
			String lab = selectedNewLanguage.getLabel();
			int index = cvLanguageComboBox.getSelectedIndex();

			if (index >= 0 && cv.setLanguageIds(index, s, l, lab)) {
				updateLanguageComboBox();
				cvLanguageComboBox.setSelectedIndex(index);				
			} else {
				showErrorDialog(ElanLocale.getString("EditCVLanguagesDialog.Error.ChangeFailed"));
			}
		} else if (source == closeButton) {
			closeDialog();
		} else if (source == cvLanguageComboBox) {
			int index = cvLanguageComboBox.getSelectedIndex();
			if (index >= 0) {
				cvNewLanguageComboBox.setSelectedIndex(getIdIndex(cv.getLongLanguageId(index)));
			}
		} else if (source == cvNewLanguageComboBox) {
		}
	}
	
	/**
	 * Find out what language info has been selected or typed by the user.
	 * Don't await a call to actionPerformed(): that happens only when they hit Enter.
	 */
	private LangInfo getSelectedNewLanguage() {
		Object selection = cvNewLanguageComboBox.getSelectedItem();
		if (selection instanceof LangInfo) {
			selectedNewLanguage = (LangInfo)selection;
		} else if (selection instanceof String) {
			selectedNewLanguage = LanguageCollection.tryParse((String)selection);
			selectedNewLanguage = LanguageCollection.validate(selectedNewLanguage);
			// Remember custom selection in list for possible re-editing.
			// Keep it as a LangInfo if possible.
			((LanguagesListModel)cvNewLanguageComboBox.getModel()).setCustomItem(
					selectedNewLanguage != null ? selectedNewLanguage
							                    : selection);
		}
		return selectedNewLanguage;
	}
	
	/**
	 * Find the index in the cvNewLanguageComboBox of the given id
	 * @return
	 */
	private int getIdIndex(String id) {
		int size = cvNewLanguageComboBox.getItemCount();
		for (int i = 0; i < size; i++) {
			Object o = cvNewLanguageComboBox.getItemAt(i);
			if (o instanceof LangInfo) {
				LangInfo li  = (LangInfo) o;
				if (id.equals(li.getId()) || id.equals(li.getLongId())) {
					return i;
				}
			}
		}
		return -1;
	}
	
    /**
     * Closes the dialog
     */
    protected void closeDialog() {
        setVisible(false);
        dispose();
    }
    
    /**
     * Extracts the languages from the CV and fills the cv language combobox.
     */
    protected void updateLanguageComboBox() {
        int nLangs = cv.getNumberOfLanguages();
        
        cvLanguageComboBox.removeAllItems();
        
        for (int i = 0; i < nLangs; i++) {
        	cvLanguageComboBox.addItem(cv.getLangInfo(i));
        }
       	cvLanguageComboBox.setSelectedIndex(0);
       	updateButtons();
    }
    
    
    /**
     * Update the "new language" combobox with a list of "all possible"
     * languages.
     * Since filling a combobox with a few thousand entries is rather slow,
     * and we have the entries in a List already anyway, just create a model
     * that adapts the List.
     */
    protected void updateNewLanguageComboBox() {

    	final List<LangInfo> languages = LanguageCollection.getLanguages();
    	
    	ComboBoxModel m = new LanguagesListModel(languages);
    	 // speeds up initial display drastically:
    	cvNewLanguageComboBox.setPrototypeDisplayValue(languages.get(0));
    	cvNewLanguageComboBox.setModel(m);
		cvNewLanguageComboBox.setSelectedIndex(getIdIndex(cv.getLongLanguageId(0)));
    }

    /**
     * Enable/disable the buttons based on the current state.
     * For instance, the last language cannot be deleted, and
     * the first language must be changed before you can add more.
     */
    protected void updateButtons() {
    	deleteButton.setEnabled(cv.getNumberOfLanguages() > 1);
    	boolean adding = cv.getNumberOfLanguages() > 1 || 
    			         !cv.getLongLanguageId(0).equals(BasicControlledVocabulary.DEFAULT_LANGUAGE_DEF);
   		addButton.setEnabled(adding);
    }
    
    /**
     * Shows a confirm (yes/no) dialog with the specified message string.
     *
     * @param message the message to display
     *
     * @return true if the user clicked OK, false otherwise
     */
    protected boolean showConfirmDialog(String message) {
        int confirm = JOptionPane.showConfirmDialog(this, message,
        		ElanLocale.getString("Message.Warning"),
                JOptionPane.YES_NO_OPTION);

        return confirm == JOptionPane.YES_OPTION;
    }

    /**
     * Shows a error dialog with the specified message string.
     *
     * @param message the message to display
     */
    protected void showErrorDialog(String message) {
        JOptionPane.showMessageDialog(this, message,
        		ElanLocale.getString("Message.Error"),
                JOptionPane.ERROR_MESSAGE);
    }

    /**
     * Use a custom model for the NewLanguageDialog to avoid duplicating the whole list
     * (about 7700 elements) element by element when adding it to the combo box.
     * We keep one extra element for a user-edited string.
     * 
     * @author olasei
     */
    private static class LanguagesListModel extends AbstractListModel implements ComboBoxModel {
    	final List<LangInfo> languages;
    	final int size;
    	Object selectedItem;
    	Object customItem;
    	
    	LanguagesListModel(List<LangInfo> languages) {
    		this.languages = languages;
    		this.size = languages.size();
    	}
    	
		@Override // AbstractListModel
		public int getSize() {
			return size + (customItem == null ? 0 : 1);
		}

		@Override // AbstractListModel
		public Object getElementAt(int index) {
			if (index < size) {
				return languages.get(index);
			} else {
				return customItem;
			}
		}

		@Override // ComboBoxModel; copied from DefaultComboBoxModel
		public void setSelectedItem(Object anItem) {
	        if ((selectedItem != null && !selectedItem.equals(anItem)) ||
	                selectedItem == null && anItem != null) {
                selectedItem = anItem;
                fireContentsChanged(this, -1, -1);
	        }
		}

		@Override // ComboBoxModel
		public Object getSelectedItem() {
			return selectedItem;
		}
		
		/**
		 * Set a user-edited item.
		 * This can be a proper LangInfo (if the user typed something that could be parsed
		 * as one) or just a String otherwise.
		 * The item is located just following the standard list of languages.
		 * @param item
		 */
		public void setCustomItem(Object item) {
			customItem = item;
			int index = size;
			if (item == null) {
	            fireIntervalRemoved(this, index, index);
			} else {
	            fireIntervalAdded(this, index, index);
			}
		}
    }
}
