package mpi.eudico.client.annotator.interlinear.edit;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JWindow;
import javax.swing.SwingUtilities;

import nl.mpi.lexan.annotyzers.helpers.Position;

import mpi.eudico.client.annotator.gui.InlineEditBox;
import mpi.eudico.client.annotator.interlinear.IGTTierType;
import mpi.eudico.client.annotator.interlinear.edit.actions.IGTEditAction;
import mpi.eudico.client.annotator.interlinear.edit.event.IGTDataModelEvent;
import mpi.eudico.client.annotator.interlinear.edit.event.IGTDataModelListener;
import mpi.eudico.client.annotator.interlinear.edit.event.SuggestionSelectionListener;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTAbstractDataModel;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTAnnotation;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTDataModel;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTSuggestionViewerModel;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTTier;
import mpi.eudico.client.annotator.interlinear.edit.render.IGTNodeRenderInfo;
import mpi.eudico.client.annotator.interlinear.edit.render.IGTViewerRenderInfo;

/**
 * A class to edit a group of annotations, in an interlinear glossed text style.
 * 
 * @author Han Sloetjes
 */
public class IGTGroupEditor extends JComponent implements MouseListener, KeyListener, IGTDataModelListener {
	private boolean debug = true;
	
	private IGTEditProvider editProvider;
	private IGTAbstractDataModel model;
	private IGTViewerRenderInfo vrInfo;
	private int row;
	private int yOff;
	
	private IGTAnnotation selectedAnn = null;
    // editing, temporary?
    private InlineEditBox editBox;
    private boolean deselectCommits = false;
    private boolean forceOpenControlledVocabulary = false;
    
	
	/**
	 * Constructor.
	 */
	public IGTGroupEditor(IGTEditProvider editProvider) {
		super();
		this.editProvider = editProvider;
		this.setLayout(null);
        editBox = new InlineEditBox(true);
        editBox.addKeyListener(this);
        
        editBox.setFont(getFont());
        editBox.setVisible(false);
        editBox.setBorder(null);
        editBox.setMargin(new Insets(0, 0, 0, 0));
        this.add(editBox);
		addMouseListener(this);
	}

	/**
	 * Sets the content for the editor, the data of one row in the viewer model.
	 * A row in the model consists of a root annotation and its dependent annotations.
	 * 
	 * @param model the row data model
	 * @param vrInfo the rendering properties of the viewer level
	 * @param row the index of the row to edit
	 * @param xOff a horizontal offset in the coordinate space of editor component 
	 * @param yOff a vertical offset in the coordinate space of editor component
	 */
	public void editIGTGroup(IGTDataModel model, IGTViewerRenderInfo vrInfo, int row,
			int xOff, int yOff) {
		if (this.model != null) {
			this.model.removeIGTDataModelListener(this);
			if (editBox.isVisible()) {
				editBox.commitEdit();// TODO let this depend on the setting in the preferences 
			}
		}
		this.model = (IGTAbstractDataModel) model;// safe cast
		this.vrInfo = vrInfo;
		this.row = row;
		this.yOff = yOff;
		
		if (this.model != null) {
			this.model.addIGTDataModelListener(this);
		}
		//removeAll();//??
		repaint();
	}
	
	/**
	 * The editor is informed of the delivery of suggestion sets.
	 * 
	 * @param svModel the viewer model containing the suggestions
	 * @param pos the position of the source annotation
	 * @param 
	 */
	public void suggestionSetDelivered(IGTSuggestionViewerModel svModel, Position pos, SuggestionSelectionListener listener) {
		if (pos != null) {
			// hier...
			int row = model.getRowIndexForTier(pos.tierId);
			if (row > -1) {
				IGTTier igtTier = model.getRowData(row);
				IGTAnnotation igtAnn = null;
				for (int i = 0; i < igtTier.getAnnotations().size(); i++) {
					igtAnn = igtTier.getAnnotations().get(i);
					if (igtAnn.getAnnotation() != null && igtAnn.getAnnotation().getBeginTimeBoundary() <= pos.beginTime
							&& igtAnn.getAnnotation().getEndTimeBoundary() >= pos.endTime) {
						break;
					}
					
					if (i == igtTier.getAnnotations().size() - 1) {
						igtAnn = selectedAnn;
					}
				}
				if (igtAnn == null) {
					igtAnn = selectedAnn;
				}
				if (igtAnn != null) {
					// found, show the suggestions below the annotation
					SuggestionSetSelector selector = new SuggestionSetSelector(svModel, vrInfo);
					
					SuggestionWindow window = new SuggestionWindow(SwingUtilities.getWindowAncestor(this), selector);
					window.setSuggestionSelectionListener(listener);
					window.setSize(300, 100);
					window.setVisible(true);

					Point gePos = this.getLocationOnScreen();
					Dimension windowSize = window.getSize();
					window.setLocation(gePos.x + vrInfo.headerWidth + igtAnn.getRenderInfo().x + (igtAnn.getRenderInfo().calcWidth / 2) - (windowSize.width / 2), 
							gePos.y + igtAnn.getRenderInfo().y + igtAnn.getRenderInfo().height + 2 * vrInfo.vertLineMargin);
					
					
				}
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		//System.out.println("Paint component...");
		Graphics2D g2d = (Graphics2D) g;
		// clear the rectangle for this row
//		if (row % 2 == 0) {
			g2d.setColor(Color.CYAN);
//		} else {
//			g2d.setColor(vrInfo.backgroundColor2);
		//}
		int width = getWidth();
		int height = getHeight();
		
		g2d.fillRect(0, 0, width, height);
		g2d.setColor(Color.BLUE);
		g2d.drawRect(0, 0, width - 1, height - 1);
		
		g2d.setColor(vrInfo.foregroundColor);
		g2d.drawLine(vrInfo.headerWidth, 0, vrInfo.headerWidth, height);
		
		// temp
		// start rendering tiers, tier labels, annotations
		int xOff = 0;//remove
		int tempY = yOff;
		IGTTier tier;
		IGTAnnotation igtAnn;
		IGTNodeRenderInfo annRender;
		
		List<IGTTier> renderedTier = new ArrayList<IGTTier>();
		for (int i = 0; i < model.getRowCount(); i++) {
			tier = model.getRowData(i);
			if (renderedTier.contains(tier)) {
				continue;
			}
			
			if (tier.isInWordLevelBlock()) {
				// tier labels
				if (tier.getRenderInfo().getYPositions() != null) {
					for (int j = 0; j < tier.getRenderInfo().getYPositions().size(); j++) {
						// TODO position the baseline of the text correctly
						tempY = yOff + tier.getRenderInfo().getYPositions().get(j) + tier.getRenderInfo().height;
						// TODO use the font from the tier render info 
						//g2d.setFont(tier.getRenderInfo().tierFont);
						g2d.drawString(model.getShortTierNameForIndex(i), xOff, tempY - 2);
					}
				} else {
					tempY = yOff + tier.getRenderInfo().y + tier.getRenderInfo().height;
					// TODO use the font from the tier render info 
					//g2d.setFont(tier.getRenderInfo().tierFont);
					g2d.drawString(model.getShortTierNameForIndex(i), xOff, tempY - 2);
				}
				
				// annotations
				for (int j = 0; j < tier.getAnnotations().size(); j++) {
					igtAnn = tier.getAnnotations().get(j);
					annRender = igtAnn.getRenderInfo();
					tempY = yOff + annRender.y + annRender.height;
					
					// test selected state
					if (igtAnn == selectedAnn) {
						g2d.setColor(Color.GREEN);
						g2d.fillRect(xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, yOff + annRender.y, 
								annRender.calcWidth, annRender.height);
						g2d.setColor(vrInfo.foregroundColor);
//						System.out.println("Green: "+ (xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x) + " "
//								+ (tempY - tier.getRenderInfo().height) + " " + igtAnn.getRenderInfo().calcWidth + 
//						" " + tier.getRenderInfo().height);
					}
					
					g2d.drawString(igtAnn.getTextValue(), xOff + vrInfo.headerWidth + annRender.x, 
							tempY - 2);// TODO position the text correctly vertically
					if (debug) {
						g2d.setColor(Color.RED);
						g2d.drawRect(xOff + vrInfo.headerWidth + annRender.x, yOff + annRender.y, 
								annRender.calcWidth, annRender.height);
						g2d.setColor(vrInfo.foregroundColor);
					}
				}
			} else {
				// other types of tiers
				tempY = yOff + tier.getRenderInfo().y + tier.getRenderInfo().height;
				g2d.drawString(model.getShortTierNameForIndex(i), xOff, tempY - 2); // position the baseline of the text correctly
				
				if (tier.getRenderInfo().numLines > 1) {
					if (tier.getAnnotations().size() >= 1) {
						igtAnn = tier.getAnnotations().get(0);
						List<String> lines = igtAnn.getRenderInfo().getWrappedLines();
						
						if (lines != null) {
							for (int a = 0; a < lines.size(); a++) {
								if (igtAnn == selectedAnn) {
									g2d.setColor(Color.GREEN);
									g2d.fillRect(xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - tier.getRenderInfo().height, 
											igtAnn.getRenderInfo().calcWidth, tier.getRenderInfo().height);
									g2d.setColor(vrInfo.foregroundColor);
								}
								
								g2d.drawString(lines.get(a), xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - 2);
								
								if (debug) {
									g2d.setColor(Color.RED);
									g2d.drawRect(xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - tier.getRenderInfo().height, 
											igtAnn.getRenderInfo().calcWidth, tier.getRenderInfo().height);
									g2d.setColor(vrInfo.foregroundColor);
								}
								
								if (a != lines.size() - 1) {
									tempY += vrInfo.vertLineMargin;
									tempY += tier.getRenderInfo().height;
								}
							}
						} 				
					} 
				} else {
					if (tier.getAnnotations().size() >= 1) {
						igtAnn = tier.getAnnotations().get(0);
						if (igtAnn == selectedAnn) {
							g2d.setColor(Color.GREEN);
							g2d.fillRect(xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - tier.getRenderInfo().height, 
									igtAnn.getRenderInfo().calcWidth, tier.getRenderInfo().height);
							g2d.setColor(vrInfo.foregroundColor);
//							System.out.println("Green: "+ (xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x) + " "
//									+ (tempY - tier.getRenderInfo().height) + " " + igtAnn.getRenderInfo().calcWidth + " " + 
//									tier.getRenderInfo().height);
							
						}
						
						g2d.drawString(igtAnn.getTextValue(), xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - 2);
						
						if (debug) {
							g2d.setColor(Color.RED);
							g2d.drawRect(xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - tier.getRenderInfo().height, 
									igtAnn.getRenderInfo().calcWidth, tier.getRenderInfo().height);
							g2d.setColor(vrInfo.foregroundColor);
						}
					}
				}
			}
			
			renderedTier.add(tier);
			tempY += vrInfo.vertLineMargin;//??

		}
	}

	/**
	 * Finds the annotation at the given location in the editor's coordinate space.
	 * TODO reimplement when the rendering information includes y coordinates of wrapped blocks/lines
	 *  
	 * @param p the location
	 * @return the IGTAnnotation or null
	 */
	private IGTAnnotation getAnnotationAtPoint(Point p) {
		if (model != null) {
			IGTTier igtTier;
			IGTAnnotation igtAnn;
			for (int i = 0; i < model.getRowCount(); i++) {
				igtTier = model.getRowData(i);
				if (igtTier.isAtY(p.y)) { //isAtY should be a quick test whether the tier has a line at the y location
					return igtTier.getAnnotationAtPoint(p);
				}
			}
		}
		
		return null;
	}
	
	/**
	 * Starts editing the specified annotation, currently by showing a popup menu.
	 * 
	 * @param igtAnn the annotation to edit
	 */
	private void editAnnotation(IGTAnnotation igtAnn) {
		if (igtAnn == null) {
			return;
		}
		if (igtAnn.getIGTTier().getType() == IGTTierType.SILENCE_DURATION ||
				igtAnn.getIGTTier().getType() == IGTTierType.SPEAKER_LABEL ||
				igtAnn.getIGTTier().getType() == IGTTierType.TIME_CODE) {
			return;
		}
		
		List<IGTEditAction> actions = editProvider.actionsForAnotation(igtAnn);
		
		if (actions != null) {
			JPopupMenu popup = new JPopupMenu();

			for (IGTEditAction igtAction : actions) {
				popup.add(igtAction);
			}

			popup.show(this, igtAnn.getRenderInfo().x + vrInfo.headerWidth, igtAnn.getRenderInfo().y + igtAnn.getRenderInfo().height);
			
			repaint();
		}
	}
	
	/**
	 * Starts editing the specified annotation in a text edit component.
	 * 
	 * @param igtAnn the annotation to text edit
	 */
	private void textEditAnnotation(IGTAnnotation igtAnn) {
		if (igtAnn == null) {
			return;
		}
		if (igtAnn.getIGTTier().getType() == IGTTierType.SILENCE_DURATION ||
				igtAnn.getIGTTier().getType() == IGTTierType.SPEAKER_LABEL ||
				igtAnn.getIGTTier().getType() == IGTTierType.TIME_CODE) {
			return;
		}
		
		// hier...
        editBox.setAnnotation(igtAnn.getAnnotation(),
                forceOpenControlledVocabulary);

//        Font f = getFontForTier(tag2d.getTier2D().getTier());
//        if (f != null) {
//        	editBox.setFont(f);
//        }
        editBox.setFont(getFont());
       	editBox.setLocation(igtAnn.getRenderInfo().x + vrInfo.headerWidth, yOff + igtAnn.getRenderInfo().y);
       	Dimension dim = new Dimension(igtAnn.getRenderInfo().calcWidth + 2, igtAnn.getRenderInfo().height); 
        editBox.configureEditor(JPanel.class, null, dim);
        // call the override minimum version of set size
        editBox.setSizeIgnoreMinimum(dim);
        editBox.setVisible(true);
        editBox.startEdit();

        forceOpenControlledVocabulary = false;
//        System.out.println("Ann: " + (igtAnn.getRenderInfo().x + vrInfo.headerWidth) + " " + igtAnn.getRenderInfo().y 
//        		+ " " + (igtAnn.getRenderInfo().calcWidth + 2) + " " + igtAnn.getRenderInfo().height);
        //System.out.println("TF: " + editBox.getBounds());
	}
	
	/**
	 * Searches the current group for the next editable (and therefore visible) annotation.
	 * By default the search is left to right, top to bottom, but this 
	 * might need to be customizable. 
	 * 
	 * @param igtAnn the current annotation, the annotation from which the search starts.
	 */
	private void selectEditNextAnnotation(IGTAnnotation igtAnn) {
		if (igtAnn == null) {
			return;
		}
		IGTAnnotation nextAnn = igtAnn.getIGTTier().getNextAnnotation(igtAnn);
		if (nextAnn != null) {
			selectedAnn = nextAnn;
			repaint();
			textEditAnnotation(nextAnn);
		} else {
			// find next visible tier
			int curRow = model.getRowIndexForTier(igtAnn.getIGTTier().getTierName());
			// loop over following tiers/rows depending on the tab direction
			int nextRow = curRow + 1;
			IGTTier nextTier;
			
			while (true) {
				if (nextRow == model.getRowCount()) {
					 nextRow = 0;
				}
				
				if (nextRow == curRow) {
					// select the first annotation unless it is the same as the current annotation
					nextAnn = igtAnn.getIGTTier().getAnnotations().get(0);// should always be safe
					break;
				}
				nextTier = model.getRowData(nextRow);
				
				if (nextTier.getType() == IGTTierType.SILENCE_DURATION ||
						nextTier.getType() == IGTTierType.SPEAKER_LABEL ||
								nextTier.getType() == IGTTierType.TIME_CODE) {
					nextRow++;
					continue;
				}
				if (nextTier.getAnnotations().size() == 0) {
					nextRow++;
				} else {
					nextAnn = nextTier.getAnnotations().get(0);
					break;
				}
			}
			
			if (nextAnn == null || nextAnn == igtAnn) {
				selectedAnn = null;
				repaint();
			} else {
				selectedAnn = nextAnn;
				repaint();
				textEditAnnotation(nextAnn);
			}
		}
	}

	/**
	 * Searches the current group for the previous editable (and therefore visible) annotation.
	 * By default the search for the previous annotation is right to left, bottom to top (but this 
	 * might need to be customizable). 
	 * 
	 * @param igtAnn the current annotation, the annotation from which the search starts.
	 */
	private void selectEditPreviousAnnotation(IGTAnnotation igtAnn) {
		if (igtAnn == null) {
			return;
		}
		IGTAnnotation prevAnn = igtAnn.getIGTTier().getPreviousAnnotation(igtAnn);
		if (prevAnn != null) {
			selectedAnn = prevAnn;
			repaint();
			textEditAnnotation(prevAnn);
		} else {
			// find previous visible tier
			int curRow = model.getRowIndexForTier(igtAnn.getIGTTier().getTierName());
			// loop over preceding tiers/rows depending on the shift+tab direction
			int prevRow = curRow - 1;
			IGTTier nextTier;
			
			while (true) {
				if (prevRow < 0) {
					 prevRow = model.getRowCount() - 1;
				}
				
				if (prevRow == curRow) {
					// select the last annotation unless it is the same as the current annotation
					int numAnn = igtAnn.getIGTTier().getAnnotations().size();
					if (numAnn > 1) {
						prevAnn = igtAnn.getIGTTier().getAnnotations().get(numAnn - 1);
					}
					break;
				}
				nextTier = model.getRowData(prevRow);
				
				if (nextTier.getType() == IGTTierType.SILENCE_DURATION ||
						nextTier.getType() == IGTTierType.SPEAKER_LABEL ||
								nextTier.getType() == IGTTierType.TIME_CODE) {
					prevRow--;
					continue;
				}
				if (nextTier.getAnnotations().size() == 0) {
					prevRow--;
				} else {
					int numAnn = nextTier.getAnnotations().size();
					prevAnn = nextTier.getAnnotations().get(numAnn - 1);
					break;
				}
			}
			
			if (prevAnn == null || prevAnn == igtAnn) {
				selectedAnn = null;
				repaint();
			} else {
				selectedAnn = prevAnn;
				repaint();
				textEditAnnotation(prevAnn);
			}
		}
	}
	/**
	 * Indirect mouse clicked event, called when a row is activated, starts editing the annotation
	 * at that point.
	 * 
	 * @param x the x position
	 * @param y the y position
	 */
	public void clickedAt(int x, int y) {
		Point p = new Point(x - vrInfo.headerWidth, y);
		IGTAnnotation actAnnotation = getAnnotationAtPoint(p);
		
		selectedAnn = actAnnotation;
		repaint();
		//editAnnotation(selectedAnn);
		textEditAnnotation(selectedAnn);
	}
	

	@Override
	public void dataModelChanged(IGTDataModelEvent event) {
		repaint();
	}
	
	@Override
	public void mouseClicked(MouseEvent me) {
        if ((SwingUtilities.isRightMouseButton(me) && (me.getButton() == MouseEvent.BUTTON1 ^ me.isMetaDown())) 
                || me.isPopupTrigger()) {
        	return;
        }
		Point p = me.getPoint();
		p.x -= vrInfo.headerWidth;
		IGTAnnotation actAnnotation = getAnnotationAtPoint(p);
		
		selectedAnn = actAnnotation;
		repaint();

		textEditAnnotation(selectedAnn);

	}

	@Override
	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mousePressed(MouseEvent me) {
        if ((SwingUtilities.isRightMouseButton(me) && (me.getButton() == MouseEvent.BUTTON1 ^ me.isMetaDown())) 
                || me.isPopupTrigger()) {
    		Point p = me.getPoint();
    		p.x -= vrInfo.headerWidth;
    		IGTAnnotation actAnnotation = getAnnotationAtPoint(p);
    		
    		selectedAnn = actAnnotation;
    		repaint();

    		editAnnotation(selectedAnn);
        }
		
	}

	@Override
	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void keyPressed(KeyEvent ke) {
		/**
		 * For now assumes the key event came from the text edit box.
		 */
		if (ke.getKeyCode() == KeyEvent.VK_TAB) {
			editBox.commitEdit();
			if (ke.isShiftDown()) {
				selectEditPreviousAnnotation(selectedAnn);
			} else {
				selectEditNextAnnotation(selectedAnn);
			}
			ke.consume();
		}
		
	}

	@Override
	public void keyReleased(KeyEvent ke) {
		if (ke.getKeyCode() == KeyEvent.VK_TAB) {
			//System.out.println("Tab release:");
			ke.consume();
		}
		//System.out.println("Key released: " + editBox.getCurrentText());
		//if (ke.getSource() == editBox) {
			int curWidth = editBox.getWidth();
			int textWidth = editBox.getCurrentTextWidth();
			//textWidth += 0;// correct for some margin space, turn into a constant
			if (textWidth != curWidth && textWidth >= selectedAnn.getRenderInfo().calcWidth) {
				Dimension dim = editBox.getSize();
				dim.width = textWidth;
				if (editBox.getX() + dim.width < getWidth()) {
					editBox.setSizeIgnoreMinimum(dim);
					editBox.revalidate();
					// update the calc width of the annotation block and positions of annotations to the right?
					IGTCalculator.updateLASRecursiveFromNode(selectedAnn, textWidth);
				}
				// check position of edit box and size of the editor as a whole
				
				repaint();
			}
		//}
	}

	/**
	 * Resizing upon a key typed work well because the typed character hasn't been added
	 * to the text document yet.
	 */
	@Override
	public void keyTyped(KeyEvent ke) {
		//System.out.println("Key typed: " + editBox.getCurrentText());
	}

	
	
}
