package mpi.eudico.client.annotator.interlinear.edit.actions;

import java.awt.event.ActionEvent;

import javax.swing.Icon;

import nl.mpi.lexan.annotyzers.Mediator;
import nl.mpi.lexan.annotyzers.helpers.Position;

import mpi.eudico.client.annotator.interlinear.edit.MediatorOutConnector;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTAnnotation;
import mpi.eudico.server.corpora.clomimpl.abstr.AbstractAnnotation;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;

/**
 * Temporary class for deleting an annotation. 
 * Should be undoable.
 * 
 * @author Han Sloetjes
 *
 */
public class IGTDeleteAction extends IGTEditAction {
	/** The mediator connector to use for starting the interlinearization action 
	 * In case of deletion the connected annotyzer might need to know of the removal? */
	protected MediatorOutConnector mediator;
	
	/**
	 * Constructor.
	 * 
	 * @param igtAnnotation the IGT annotation
	 * @param mediator the mediator via which the connected annotyzer can be informed of deletion
	 */
	public IGTDeleteAction(IGTAnnotation igtAnnotation, MediatorOutConnector mediator) {
		super(igtAnnotation);
		this.mediator = mediator;
	}

	/**
	 * Constructor.
	 * 
	 * @param igtAnnotation the IGT annotation
	 * @param mediator the mediator via which the connected annotyzer can be informed of deletion
	 * @param label the text for the action 
	 */	
	public IGTDeleteAction(IGTAnnotation igtAnnotation, MediatorOutConnector mediator, String label) {
		super(igtAnnotation, label);
		this.mediator = mediator;
	}
	
	/**
	 * Constructor.
	 * 
	 * @param igtAnnotation the IGT annotation
	 * @param mediator the mediator via which the connected annotyzer can be informed of deletion
	 * @param label the text for the action
	 * @param icon an icon for the action 
	 */
	public IGTDeleteAction(IGTAnnotation igtAnnotation, MediatorOutConnector mediator, String label, 
			Icon icon) {
		super(igtAnnotation, label, icon);
		this.mediator = mediator;
		
	}

	/**
	 * 
	 */
	@Override
	public void actionPerformed(ActionEvent arg0) {
		if (igtAnnotation != null && mediator != null) {
			Position pos = new Position();
			pos.tierId = igtAnnotation.getAnnotation().getTier().getName();
			pos.beginTime = igtAnnotation.getAnnotation().getBeginTimeBoundary();
			pos.endTime = igtAnnotation.getAnnotation().getEndTimeBoundary();
			
			// this might need changing
			mediator.annotationDeleted(pos);
			
//			AbstractAnnotation aa = igtAnnotation.getAnnotation();
//			if (aa != null) {
//				TierImpl tier = (TierImpl) aa.getTier();
//				tier.removeAnnotation(aa);// generates event
//			}
		}

	}

}
