package mpi.eudico.client.annotator.interlinear.edit.render;

import java.awt.Color;
import java.awt.Font;

import java.util.HashMap;
import java.util.Map;

import mpi.eudico.client.annotator.Constants;

public class IGTViewerRenderInfo extends IGTRenderInfo {

	public int vertRowMargin; // the empty space between two rows (== root annotation + depending annotations)
	public int vertLineMargin; // the empty space between two lines of text (between two tiers)
	public int headerWidth; // width of tier label area
	
	public int whitespaceWidth;
	// temp
	public int tierHeight;
	
	public Color foregroundColor = Constants.DEFAULTFOREGROUNDCOLOR;
	public Color backgroundColor = Color.WHITE;
	public Color backgroundColor2 = new Color(230, 230, 230);
	
	// font information
	private Map<String, Font> fontMap;

	/**
	 * No-arg constructor.
	 */
	public IGTViewerRenderInfo() {
		super();
		 fontMap = new HashMap<String, Font>();
	}
	
	/**
	 * Returns the font for the particular tier.
	 * 
	 * @param tierName the name of the tier
	 * @return the font or null
	 */
	public Font getFontForTier(String tierName) {
		return fontMap.get(tierName);
	}
	
	/**
	 * Sets the font for the specified tier.
	 * 
	 * @param tierName the name of the tier
	 * @param f the font
	 */
	public void setFontForTier(String tierName, Font f) {
		fontMap.put(tierName, f);
	}
	
	/**
	 * Returns the size of the font for this tier as an int.
	 * 
	 * @param tierName the tier name
	 * @return the size of font as an int, defaults to 12 if there is no mapping
	 */
	public int getFontSizeForTier(String tierName) {
		Font f = fontMap.get(tierName);
		
		if (f != null) {
			return f.getSize();
		}
		
		return 12;
	}
	
	/**
	 * Returns the height for a tier "line". This height is based on the size of the font for
	 * the tier.
	 * 
	 * @param tierName the name of the tier
	 * @return the line height for a tier, the font size plus some extra space, defaults to 14
	 */
	public int getHeightForTier(String tierName) {
		Font f = fontMap.get(tierName);
		
		if (f != null) {
			return f.getSize() + 2;//  TODO implement something more sensible 
		}
		
		return 14;
	}
	
}
