package mpi.eudico.client.annotator.layout;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.border.SoftBevelBorder;

import nl.mpi.lexan.annotyzers.ControllerImpl;
import nl.mpi.lexan.annotyzers.Mediator;
import nl.mpi.lexan.annotyzers.MediatorImpl;
import nl.mpi.lexan.annotyzers.helpers.Information;

import mpi.eudico.client.annotator.ElanLayoutManager;
import mpi.eudico.client.annotator.Preferences;
import mpi.eudico.client.annotator.ViewerManager2;
import mpi.eudico.client.annotator.gui.ResizeComponent;
import mpi.eudico.client.annotator.interlinear.edit.AnnotConfig;
import mpi.eudico.client.annotator.interlinear.edit.AnnotConfigPanel;
import mpi.eudico.client.annotator.interlinear.edit.InterlinearEditor;
import mpi.eudico.client.annotator.interlinear.edit.InterlinearMediator;
import mpi.eudico.client.annotator.interlinear.edit.AnnotTypeConfig;
import mpi.eudico.server.corpora.clom.Transcription;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;

public class InterlinearizationManager implements ModeLayoutManager{
	
	private ViewerManager2 viewerManager;
	private ElanLayoutManager layoutManager;
	private Container container;
	// store for restoring orig values
	int origMediaAreaWidth = ElanLayoutManager.MASTER_MEDIA_WIDTH;
	int origMediaAreaHeight = ElanLayoutManager.MASTER_MEDIA_HEIGHT;
	// components
	private ResizeComponent leftRightResizer;
	private ResizeComponent topBottomResizer;
	
	private JPanel leftPanel;
	private AnnotConfigPanel topPanel;
	private InterlinearEditor interPanel;
	
	private InterlinearMediator mediator;

	/**
	 * Creates an instance of the InterlinearizationManager
	 * 
	 * @param viewerManager
	 * @param elanLayoutManager
	 */
    public InterlinearizationManager(ViewerManager2 viewerManager, ElanLayoutManager elanLayoutManager) {
    	super();
        this.viewerManager = viewerManager;
        this.layoutManager = elanLayoutManager;
        
        mediator = new InterlinearMediator(this);
        ControllerImpl controller = new ControllerImpl();
        String loc = System.getProperty("LexanPluginDir");
        if (loc != null) {
        	System.out.println("Plugin directory for Lexan: " + loc);
	        ArrayList<String> locs = new ArrayList<String>(1);
	        locs.add(loc);
	        controller.initAnnotyzers(locs);
        } else {
        	System.out.println("No plugin directory specified for Lexan");
        }
        Mediator lexanMediator = new MediatorImpl(mediator, controller);

        mediator.setMediator(lexanMediator);
        container = layoutManager.getContainer();
        leftPanel = new JPanel();
        topPanel = new AnnotConfigPanel(this);
        interPanel = new InterlinearEditor(this);
        // temp this might needed to be done elsewhere, in the viewer manager
        viewerManager.getTranscription().addACMEditListener(interPanel);
    }

    /**
     * Convenient access to the mediator.
     * @return
     */
	public Mediator getMediator() {
		return mediator;
	}
	
	/**
	 * Convenient access to the transcription.
	 * @return
	 */
	public Transcription getTranscription() {
		return viewerManager.getTranscription();
	}
	
	/**
	 * Temp: grant access to the editor component.
	 * 
	 * @return
	 */
	public InterlinearEditor getInterEditor() {
		return interPanel;
	}
	
	public AnnotConfigPanel getConfigPanel() {
		return topPanel;
	}
	
	public void add(Object object) {
		// TODO Auto-generated method stub
		
	}
	
	public void remove(Object object) {
		// TODO Auto-generated method stub
		
	}
	
	public void doLayout() {
		int containerWidth 		= container.getWidth();
	    int containerHeight 	= container.getHeight();
	    int mediaAreaWidth 		= layoutManager.getMediaAreaWidth();
	    int mediaAreaHeight     = layoutManager.getMediaAreaHeight();
	    int visibleMediaX 		= ElanLayoutManager.CONTAINER_MARGIN;
		// resize divider       
        int lrDivX = visibleMediaX + mediaAreaWidth + ElanLayoutManager.CONTAINER_MARGIN;
        int lrDivY = containerHeight - leftRightResizer.getPreferredSize().height; 

        leftRightResizer.setBounds(lrDivX, lrDivY, leftRightResizer.getPreferredSize().width,  
        		leftRightResizer.getPreferredSize().height);
		
        int tbDivX = lrDivX + leftRightResizer.getWidth();
        int tbDivY = 2 * ElanLayoutManager.CONTAINER_MARGIN + mediaAreaHeight;
        
        topBottomResizer.setBounds(tbDivX, tbDivY, containerWidth - tbDivX, topBottomResizer.getPreferredSize().height);
        
        leftPanel.setBounds(ElanLayoutManager.CONTAINER_MARGIN, ElanLayoutManager.CONTAINER_MARGIN, 
        		lrDivX - ElanLayoutManager.CONTAINER_MARGIN, containerHeight - (2 * ElanLayoutManager.CONTAINER_MARGIN));
        
        int rightCompX = tbDivX + ElanLayoutManager.CONTAINER_MARGIN;
        int rightCompW = containerWidth - rightCompX - ElanLayoutManager.CONTAINER_MARGIN;
        topPanel.setBounds(rightCompX, ElanLayoutManager.CONTAINER_MARGIN, 
        		rightCompW, tbDivY - ElanLayoutManager.CONTAINER_MARGIN);
        
        int interCompY = tbDivY + topBottomResizer.getHeight() + ElanLayoutManager.CONTAINER_MARGIN;
        int interCompH = containerHeight - interCompY - ElanLayoutManager.CONTAINER_MARGIN;
        interPanel.setBounds(rightCompX, interCompY, rightCompW, interCompH);
	}
	
	public void updateLocale() {
		topPanel.updateLocale();		
	}
	
	public void clearLayout() {
		storePreferences();
		viewerManager.getTranscription().removeACMEditListener(interPanel);
		// remove all interlinear components
		container.remove(leftRightResizer);
		container.remove(topBottomResizer);
        container.remove(leftPanel);
        container.remove(topPanel);
        container.remove(interPanel);
		// restore some values
		layoutManager.setMediaAreaWidth(origMediaAreaWidth);
		layoutManager.setMediaAreaHeight(origMediaAreaHeight);
		// add the video players
		PlayerLayoutModel model = null;
		List visuals = layoutManager.getPlayerList();
		for (int i = 0; i < visuals.size(); i++) {
			model = (PlayerLayoutModel) visuals.get(i);
			if (model.isVisual() && model.isAttached()) {
				container.add(model.visualComponent);
			}
		}

		container.repaint();
	}
	
	private void storePreferences() {
//		if (mediator != null && mediator.getConfigurations() != null 
//				&& mediator.getConfigurations().size() > 0) {			
//			List<String> confList = new ArrayList<String>();				
//			HashMap<String, List<String>> excludedTierMap = new HashMap<String, List<String>>();
//			for (AnnotConfig ac : mediator.getConfigurations()) {
//				confList.add(ac.getAnnotId().name + ", " + ac.getSource() + ", " + ac.getDest()+", " + ac.isTypeMode());
//					
//				if(ac.isTypeMode()){
//					List<TierImpl> sourceTiers = viewerManager.getTranscription().getTiersWithLinguisticType(ac.getSource());
//					List<TierImpl> desTiers = viewerManager.getTranscription().getTiersWithLinguisticType(ac.getDest());
//					
//					HashMap<String, List<String>> tierMap = new HashMap<String, List<String>>();
//					List<String> destList;
//					
//					for(int i= 0; i < desTiers.size(); i++){
//						if(sourceTiers.contains (desTiers.get(i).getParentTier())){
//							
//							if(tierMap.containsKey(desTiers.get(i).getParentTier().getName())){
//								destList = tierMap.get(desTiers.get(i).getParentTier().getName());
//								destList.add(desTiers.get(i).getName());
//							} else {
//								destList = new ArrayList<String>();
//								destList.add(desTiers.get(i).getName());
//								tierMap.put(desTiers.get(i).getParentTier().getName(), destList);		
//							}				
//						}
//					}
//					
//					for (AnnotConfig tc :((AnnotTypeConfig)ac).getTierConfigurations()) {
//						if(tierMap.containsKey(tc.getSource())){
//							destList = tierMap.get(tc.getSource());
//							if(destList.contains(tc.getDest())){
//								destList.remove(tc.getDest());
//								if(destList.size() == 0){
//									tierMap.remove(tc.getSource());
//								}
//							}
//						}
//					}
//					
//					if(tierMap.size() > 0){
//						excludedTierMap.putAll(tierMap);
//					}
//				}
//			}			
//			
//			AnnotConfig active = interPanel.getActiveConfiguration();
//			if( active != null){
//				String activeConf = active.getAnnotId().name+", " + active.getSource() +", "+ active.getDest()+", " + active.isTypeMode();
//				Preferences.set("InterlinearizationConfigurations.ActiveConfiguration", activeConf, viewerManager.getTranscription(), false, false);
//			}
//			
//			Preferences.set("InterlinearizationConfigurations", confList, viewerManager.getTranscription(), false, false);
//			Preferences.set("InterlinearizationConfigurations.ExcludedTiers", excludedTierMap, viewerManager.getTranscription(), false, false);
//		}
	}
	
	private void readPreferences() {		
//		Object confObj = Preferences.get("InterlinearizationConfigurations", viewerManager.getTranscription());		
//		Object activeConfObj = Preferences.get("InterlinearizationConfigurations.ActiveConfiguration", viewerManager.getTranscription());
//		Object excludedObj = Preferences.get("InterlinearizationConfigurations.ExcludedTiers", viewerManager.getTranscription());		
//		
//		AnnotConfig activeConf = null;
//		String activeTokens[] = null;
//		
//		if(activeConfObj != null){
//			activeTokens = ((String)activeConfObj).split(", ");
//		}
//		
//		HashMap<String, List<String>> excludedTiersMap = new HashMap<String, List<String>>();
//		
//		if(excludedObj instanceof HashMap){
//			excludedTiersMap = (HashMap) excludedObj;			
//			
//			
////			for (Object item : excludedTiers) {
////				if (item instanceof String) {
////					String conf = (String) item;
////					String[] tokens = conf.split(", ");
////					if (tokens.length == 2) {	
////						if(tierMap.containsKey(tokens[0])){
////							destList = tierMap.get(tokens[0]);
////							if(!destList.contains(tokens[1])){
////								destList.add(tokens[1]);
////							}							
////						} else {
////							destList = new ArrayList<String>();
////							destList.add(tokens[1]);
////							
////							tierMap.put(tokens[0], destList);
////						}
////					}
////				}	
////			}
//		}
//		
//		if (confObj instanceof List) {
//			List<Information> tyzers = mediator.listAnnotyzers();
//			List stored = (List) confObj;
//			
//			List<TierImpl> sourceTiers;
//			List<TierImpl> destTiers;
//			AnnotConfig ac;
//			AnnotTypeConfig atc; 
//			
//			for (Object item : stored) {
//				if (item instanceof String) {
//					String conf = (String) item;
//					String[] tokens = conf.split(", ");
//					if (tokens.length == 4) {
//						
//						Information info = null;
//						String name = tokens[0];
//						
//						if (tyzers != null) {
//							for (Information in : tyzers) {
//								if (name.equals(in.name)) {
//									info = in;
//									break;
//								}
//							}
//						}
//						
//						if(tokens[3].equals(Boolean.toString(true))){
//							sourceTiers = viewerManager.getTranscription().getTiersWithLinguisticType(tokens[1]);
//							destTiers = viewerManager.getTranscription().getTiersWithLinguisticType(tokens[2]);
//							if(sourceTiers == null  || destTiers == null){
//								continue;
//							}
//							
//							atc = new AnnotTypeConfig(info, tokens[1], tokens[2]);
//							mediator.addConfig(atc);
//							
//							for(int i= 0; i< destTiers.size(); i++){							
//								if(sourceTiers.contains(((TierImpl)destTiers.get(i)).getParentTier())){		
//									String source = ((TierImpl)destTiers.get(i)).getParentTier().getName();	
//									if(!excludedTiersMap.containsKey(source) || 
//											!excludedTiersMap.get(source).contains(destTiers.get(i).getName())){
//										ac = new AnnotConfig(info, source, destTiers.get(i).getName());
//										atc.addTierConf(ac);
//										
//										if(activeConf == null && isActiveConf(activeTokens, ac)){
//											activeConf = ac;
//										}										
//									}
//								}
//							}
//							
//							if(activeConf == null && isActiveConf(activeTokens, atc)){
//								activeConf = atc;
//							}
//							
//						} else {
//							if(viewerManager.getTranscription().getTierWithId(tokens[1]) == null ||
//									viewerManager.getTranscription().getTierWithId(tokens[2]) == null){
//								continue;
//							}							
//							
//							ac = new AnnotConfig(info, tokens[1], tokens[2]);
//							mediator.addConfig(ac);
//							
//							if(activeConf == null && isActiveConf(activeTokens, ac)){
//								activeConf = ac;
//							}
//						}
//					}
//				}
//			}
//		}
//		
//		if(interPanel != null && activeConf != null){
//			interPanel.setActiveConfiguration(activeConf);
//		}
//			
//		
//		if (topPanel != null && mediator.getConfigurations().size() > 0) {
//			topPanel.configsChanged();
//		}
	}
	
	/**
	 * Checks if the give conf is the active configuration
	 */
	private boolean isActiveConf(String[] tokens, AnnotConfig ac){
		if(tokens != null && tokens.length == 4){
			if(tokens[0].equals(ac.getAnnotId().name) &&
					tokens[1].equals(ac.getSource()) &&
					tokens[2].equals(ac.getDest()) &&
					tokens[3].equals(Boolean.toString(ac.isTypeMode()))){
				return true;
			}
		}		
		return false;
	}
	
	public void initComponents() {
		origMediaAreaWidth = layoutManager.getMediaAreaWidth();
		origMediaAreaHeight = layoutManager.getMediaAreaHeight();
		// remove remaining video player components
		PlayerLayoutModel model = null;
		List visuals = layoutManager.getPlayerList();
		for (int i = 0; i < visuals.size(); i++) {
			model = (PlayerLayoutModel) visuals.get(i);
			if (model.isVisual() && model.isAttached()) {
				container.remove(model.visualComponent);
			}
		}
		// add mode specific components
        // divider component
        leftRightResizer = new ResizeComponent(layoutManager, SwingConstants.HORIZONTAL);
        leftRightResizer.setBorder(new SoftBevelBorder(SoftBevelBorder.RAISED));
        leftRightResizer.setPreferredSize(new Dimension(8,container.getHeight()));
        Component n = leftRightResizer.getComponent(0);
        leftRightResizer.remove(n);
        leftRightResizer.setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.anchor = gbc.PAGE_END;
        gbc.weighty = 1.0; 
        leftRightResizer.add(n, gbc);
        
    	container.add(leftRightResizer);
    	
    	topBottomResizer = new ResizeComponent(layoutManager, SwingConstants.VERTICAL);
    	topBottomResizer.setBorder(new SoftBevelBorder(SoftBevelBorder.RAISED));
    	topBottomResizer.setPreferredSize(new Dimension(container.getWidth(), 8));
        n = topBottomResizer.getComponent(0);
        topBottomResizer.remove(n);
        topBottomResizer.setLayout(new GridBagLayout());
        gbc = new GridBagConstraints();
        gbc.anchor = gbc.PAGE_END;
        gbc.weightx = 1.0; 
        topBottomResizer.add(n, gbc);
        
        container.add(topBottomResizer);
        
        leftPanel.setBackground(new Color(220, 220, 255));
        //topPanel.setBackground(new Color(220, 255, 220));
        //interPanel.setBackground(Color.WHITE);
        container.add(leftPanel);
        container.add(topPanel);
        container.add(interPanel);
        readPreferences();
        doLayout();
	}

	public void enableOrDisableMenus(boolean enabled) {
		// TODO Auto-generated method stub
		
	}

	public void detach(Object object) {
		// TODO Auto-generated method stub
		
	}
	
	public void attach(Object object) {
		// TODO Auto-generated method stub
		
	}

	public void preferencesChanged() {
		// TODO Auto-generated method stub
		
	}
	
	public void cleanUpOnClose() {
		// temporary
		clearLayout();	
	}

	public void shortcutsChanged() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void createAndAddViewer(String viewerName) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public boolean destroyAndRemoveViewer(String viewerName) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public void isClosing() {
		// TODO Auto-generated method stub
		
	}
} 


