package mpi.eudico.client.annotator.prefs.gui;


import javax.swing.JComboBox;

import mpi.eudico.client.annotator.prefs.RecentLanguages;
import mpi.eudico.util.multilangcv.LangInfo;
import mpi.eudico.util.multilangcv.LanguageCollection;

/**
 * A pre-filled combobox which somehow knows what languages have been in use
 * recently, and fills itself with those.
 * <p>
 * The list is also stored in a private preferences file.<br/>
 * (Maybe a more compact serialization would be nice, but extending the Preferences storage
 * would be incompatible with older Elan versions, which cannot read it.)
 * 
 * @author olasei
 */

@SuppressWarnings("serial")
public class RecentLanguagesBox extends JComboBox {
	public RecentLanguagesBox(String initialValue) {
		super();
		
		boolean selected = false;	
		this.removeAllItems();
		for (LangInfo li: RecentLanguages.getInstance().getRecentLanguages()) {
			if (li != null) {
				addItem(li);
				if (li.getLongId().equals(initialValue)) {
					setSelectedItem(li);
					selected = true;
				}
			}
		}			
		if (!selected) {
			// The current preference wasn't in the list; add it so we don't lose it.
			setEditable(true);
			setSelectedItem(initialValue);
		}
	}
	
	public String getText() {
		Object obj = getSelectedItem();
		if (obj instanceof LangInfo) {
			LangInfo li = (LangInfo) obj;
			if (li != null) {
				return li.getLongId();
			}
		} else if (obj instanceof String) {
			String text = (String)obj;
			LangInfo li = LanguageCollection.tryParse(text);
			if (li != null) {
				return li.getLongId();
			}
			return text;
		}
		return "";
	}
}
